<?xml version="1.0" encoding="utf-8"?>
<!--
    Copyright (c) Members of the EGEE Collaboration. 2004-2006.
    See http://public.eu-egee.org/partners/ for details on 
    the copyright holders.
    For license conditions see the license file or
    http://eu-egee.org/license.html

    Authors: 
         Akos Frohner <Akos.Frohner@cern.ch>

    This XSL file converts a documented WSDL file into 
    a DocBook formatted interface documentation, which 
    can be included into a larger document.
    The toplevel element is a sect1 section.

    Note that DocBook documents before V5 cannot contain
    namespaced elements, so you may have to strip them, 
    if they were present in the original WSDL file. For
    example like this:

    xsltproc wsdl-to-docbook.xsl example.wsdl | \
      perl -p -e 's/<db:/</g; s/<\/db:/<\//g; s/<(\w+)\s+xmlns:[^>]+>/<$1>/g' -
-->
<xsl:stylesheet
  xmlns:wsdd="http://xml.apache.org/axis/wsdd/"
  xmlns:xsl="http://www.w3.org/1999/XSL/Transform" 
  xmlns:wsdl="http://schemas.xmlsoap.org/wsdl/"
  xmlns:soap="http://schemas.xmlsoap.org/soap/encoding/"
  xmlns:xmlschema="http://www.w3.org/2001/XMLSchema"
  version='1.0'>
<xsl:output method="xml" version="1.0" encoding="UTF-8" indent="yes"/>

<xsl:template match="/">
    <sect1>
        <title>Interfaces</title>

    <!-- toplevel summaries -->
    <sect2>
        <title>Summary</title>
        <sect3>
            <title>Complex Types</title>
            <itemizedlist>
                <!-- complex types can be unwrapped -->
                <xsl:for-each select="/wsdl:definitions/wsdl:types/xmlschema:schema/xmlschema:complexType">
                    <listitem><para><link linkend="{@name}"><xsl:value-of select="@name"/></link></para></listitem>
                </xsl:for-each>
                <!-- or can be wrapped into an element to follow WS-I conventions -->
                <xsl:for-each select="/wsdl:definitions/wsdl:types/xmlschema:schema/xmlschema:element[./xmlschema:complexType]">
                    <listitem><para><link linkend="{@name}"><xsl:value-of select="@name"/></link></para></listitem>
                </xsl:for-each>
            </itemizedlist>
        </sect3>
        <sect3>
            <title>Port Types</title>
            <itemizedlist>
                <xsl:for-each select="/wsdl:definitions/wsdl:portType">
                    <listitem><para><link linkend="{@name}"><xsl:value-of select="@name"/></link></para></listitem>
                </xsl:for-each>
            </itemizedlist>
        </sect3>
    </sect2>

    <!-- detailed descriptions -->
    <xsl:apply-templates select="/wsdl:definitions/wsdl:types/xmlschema:schema/xmlschema:complexType"/>
    <xsl:apply-templates select="/wsdl:definitions/wsdl:types/xmlschema:schema/xmlschema:element[./xmlschema:complexType]"/>
    <xsl:apply-templates select="/wsdl:definitions/wsdl:portType"/>

    </sect1>
</xsl:template>

<!-- ======================================================================== -->
<!-- complexType -->
<!-- ======================================================================== -->

<xsl:template match="xmlschema:complexType">
    <sect2 id="{@name}">
        <title><xsl:value-of select="@name"/></title>
        <classsynopsis>
            <ooclass>
            <classname><xsl:value-of select="@name"/></classname>
            </ooclass>
            <xsl:apply-templates select="xmlschema:sequence/xmlschema:element" mode="summary">
                <xsl:with-param name="complexTypeName"><xsl:value-of select="@name"/></xsl:with-param>
            </xsl:apply-templates>
        </classsynopsis>
    </sect2>
    <xsl:apply-templates select="xmlschema:sequence/xmlschema:element" mode="details">
        <xsl:with-param name="complexTypeName"><xsl:value-of select="@name"/></xsl:with-param>
    </xsl:apply-templates>
</xsl:template>

<xsl:template match="xmlschema:element[./xmlschema:complexType]">
    <sect2 id="{@name}">
        <title><xsl:value-of select="@name"/></title>
        <classsynopsis>
            <ooclass>
            <classname><xsl:value-of select="@name"/></classname>
            </ooclass>
            <xsl:apply-templates select="xmlschema:complexType/xmlschema:sequence/xmlschema:element" mode="summary">
                <xsl:with-param name="complexTypeName"><xsl:value-of select="@name"/></xsl:with-param>
            </xsl:apply-templates>
        </classsynopsis>
    </sect2>
    <xsl:apply-templates select="xmlschema:complexType/xmlschema:sequence/xmlschema:element" mode="details">
        <xsl:with-param name="complexTypeName"><xsl:value-of select="@name"/></xsl:with-param>
    </xsl:apply-templates>
</xsl:template>

<xsl:template match="xmlschema:element" mode="summary">
    <xsl:param name="complexTypeName"/>
    <fieldsynopsis>
        <type><xsl:value-of select="substring-after(@type, ':')"/></type>
        <!-- <varname><link linkend="{$complexTypeName}.{@name}"><xsl:value-of select="@name"/></link></varname> -->
        <varname><xsl:value-of select="@name"/></varname>
    </fieldsynopsis>
</xsl:template>

<xsl:template match="xmlschema:element" mode="details">
    <xsl:param name="complexTypeName"/>
    <sect3 id="{$complexTypeName}.{@name}">
        <title><xsl:value-of select="@name"/></title>
        <fieldsynopsis>
            <type><xsl:value-of select="substring-after(@type, ':')"/></type>
            <varname><xsl:value-of select="@name"/></varname>
        </fieldsynopsis>
        <!-- copy the text into a simple paragraph, if not structured -->
        <xsl:if test="count(xmlschema:annotation/xmlschema:documentation/*) = 0">
            <para><xsl:copy-of select="xmlschema:annotation/xmlschema:documentation/text()"/></para>
        </xsl:if>
        <!-- will be empty loop, if there are no nodes -->
        <xsl:for-each select="xmlschema:annotation/xmlschema:documentation/*">
            <xsl:copy-of select="."/>
        </xsl:for-each>
    </sect3>
</xsl:template>

<!-- ======================================================================== -->
<!-- portType -->
<!-- ======================================================================== -->

<xsl:template match="wsdl:portType">
    <sect2 id="{@name}">
        <title><xsl:value-of select="@name"/></title>
        <classsynopsis>
            <ooclass>
            <classname><xsl:value-of select="@name"/></classname>
            </ooclass>
            <xsl:apply-templates select="wsdl:operation" mode="summary">
                <xsl:with-param name="portTypeName"><xsl:value-of select="@name"/></xsl:with-param>
            </xsl:apply-templates>
        </classsynopsis>
        <!-- copy the text into a simple paragraph, if not structured -->
        <xsl:if test="count(wsdl:documentation/*) = 0">
            <para><xsl:copy-of select="wsdl:documentation/text()"/></para>
        </xsl:if>
        <!-- will be empty loop, if there are no nodes -->
        <xsl:for-each select="wsdl:documentation/*">
            <xsl:copy-of select="."/>
        </xsl:for-each>
    </sect2>
    <xsl:apply-templates select="wsdl:operation" mode="details">
        <xsl:with-param name="portTypeName"><xsl:value-of select="@name"/></xsl:with-param>
    </xsl:apply-templates>
</xsl:template>

<xsl:template match="wsdl:operation" mode="summary">
    <xsl:param name="portTypeName"/>
    <xsl:variable name="input" select="wsdl:input/@name"/>
    <xsl:variable name="output" select="wsdl:output/@name"/>
    <methodsynopsis>
        <type><xsl:apply-templates select="//wsdl:message[@name=$output]" mode="methodtype"/></type>
        <methodname><xsl:value-of select="@name"/></methodname>
        <xsl:apply-templates select="//wsdl:message[@name=$input]" mode="methodparam"/>
        <xsl:for-each select="wsdl:fault">
            <exceptionname><xsl:value-of select="@name"/></exceptionname>
        </xsl:for-each>
    </methodsynopsis>
</xsl:template>

<xsl:template match="wsdl:operation" mode="details">
    <xsl:param name="portTypeName"/>
    <xsl:variable name="input" select="wsdl:input/@name"/>
    <xsl:variable name="output" select="wsdl:output/@name"/>
    <sect3 id="{$portTypeName}.{@name}">
        <title><xsl:value-of select="@name"/></title>
        <methodsynopsis>
            <type><xsl:apply-templates select="//wsdl:message[@name=$output]" mode="methodtype"/></type>
            <methodname><xsl:value-of select="@name"/></methodname>
            <xsl:apply-templates select="//wsdl:message[@name=$input]" mode="methodparam"/>
            <xsl:for-each select="wsdl:fault">
                <exceptionname><xsl:value-of select="@name"/></exceptionname>
            </xsl:for-each>
        </methodsynopsis>

        <!-- method parameters -->
        <xsl:if test="count(//wsdl:message[@name=$input]/wsdl:part) &gt; 0">
            <formalpara><title>Parameters:</title>
            <variablelist>
            <xsl:for-each select="//wsdl:message[@name=$input]/wsdl:part">
                <varlistentry><term><xsl:value-of select="@name"/></term>
                    <listitem>
                    <!-- copy the text into a simple paragraph, if not structured -->
                    <xsl:if test="count(wsdl:documentation/*) = 0">
                        <para><xsl:copy-of select="wsdl:documentation/text()"/></para>
                    </xsl:if>
                    <!-- will be empty loop, if there are no nodes -->
                    <xsl:for-each select="wsdl:documentation/*">
                        <xsl:copy-of select="."/>
                    </xsl:for-each>
                    </listitem>
                </varlistentry>
            </xsl:for-each>
            </variablelist>
            </formalpara>
        </xsl:if>

        <xsl:if test="count(//wsdl:message[@name=$output]/wsdl:part) &gt; 0">
            <formalpara><title>Returns:</title>
            <xsl:for-each select="//wsdl:message[@name=$output]/wsdl:part">
                    <!-- copy the text into a simple paragraph, if not structured -->
                    <xsl:if test="count(wsdl:documentation/*) = 0">
                        <para><xsl:copy-of select="wsdl:documentation/text()"/></para>
                    </xsl:if>
                    <!-- will be empty loop, if there are no nodes -->
                    <xsl:for-each select="wsdl:documentation/*">
                        <xsl:copy-of select="."/>
                    </xsl:for-each>
            </xsl:for-each>
            </formalpara>
        </xsl:if>

        <xsl:if test="count(wsdl:fault) &gt; 0">
            <formalpara><title>Throws:</title>
            <variablelist>
            <xsl:for-each select="wsdl:fault">
                <varlistentry><term><xsl:value-of select="@name"/></term>
                    <listitem>
                    <!-- copy the text into a simple paragraph, if not structured -->
                    <xsl:if test="count(wsdl:documentation/*) = 0">
                        <para><xsl:copy-of select="wsdl:documentation/text()"/></para>
                    </xsl:if>
                    <!-- will be empty loop, if there are no nodes -->
                    <xsl:for-each select="wsdl:documentation/*">
                        <xsl:copy-of select="."/>
                    </xsl:for-each>
                    </listitem>
                </varlistentry>
            </xsl:for-each>
            </variablelist>
            </formalpara>
        </xsl:if>

        <!-- copy the text into a simple paragraph, if not structured -->
        <xsl:if test="count(wsdl:documentation/*) = 0">
            <para><xsl:copy-of select="wsdl:documentation/text()"/></para>
        </xsl:if>
        <!-- will be empty loop, if there are no nodes -->
        <xsl:for-each select="wsdl:documentation/*">
            <xsl:copy-of select="."/>
        </xsl:for-each>
    </sect3>
</xsl:template>

<!-- methodtype -->

<xsl:template match="wsdl:message" mode="methodtype">
    <xsl:apply-templates select="wsdl:part" mode="methodtype"/>
</xsl:template>

<xsl:template match="wsdl:part[starts-with(@type, 'tns') or starts-with(@type, 'impl')]" mode="methodtype">
    <xsl:variable name="type" select="substring-after(@type, ':')"/>
    <xsl:if test="starts-with($type, 'ArrayOf')">
        <xsl:value-of select="substring-after ( substring-after ( $type, '_' ), '_')"/>
        <xsl:text>[]</xsl:text>
    </xsl:if>
    <xsl:if test="not(starts-with($type, 'ArrayOf'))"><xsl:value-of select="$type"/></xsl:if>
</xsl:template>

<xsl:template match="wsdl:part[starts-with(@element, 'tns') or starts-with(@element, 'impl')]" mode="methodtype">
    <xsl:variable name="type" select="substring-after(@element, ':')"/>
    <xsl:if test="starts-with($type, 'ArrayOf')">
        <xsl:value-of select="substring-after ( substring-after ( $type, '_' ), '_')"/>
        <xsl:text>[]</xsl:text>
    </xsl:if>
    <xsl:if test="not(starts-with($type, 'ArrayOf'))"><xsl:value-of select="$type"/></xsl:if>
</xsl:template>

<xsl:template match="wsdl:part[starts-with(@type, 'xsd:')]" mode="methodtype">
    <xsl:value-of select="substring-after(@type, ':')"/>
</xsl:template>

<!-- methodparam -->

<xsl:template match="wsdl:message" mode="methodparam">
    <xsl:apply-templates select="wsdl:part" mode="methodparam"/>
</xsl:template>

<xsl:template match="wsdl:part[starts-with(@type, 'tns') or starts-with(@type, 'impl')]" mode="methodparam">
    <methodparam>
        <type>
            <xsl:variable name="type" select="substring-after(@type, ':')"/>
            <xsl:if test="starts-with($type, 'ArrayOf')">
                <xsl:value-of select="substring-after ( substring-after ( $type, '_' ), '_')"/>
                <xsl:text>[]</xsl:text>
            </xsl:if>
            <xsl:if test="not(starts-with($type, 'ArrayOf'))"><xsl:value-of select="$type"/></xsl:if>
        </type>
        <parameter><xsl:value-of select="@name"/></parameter>
    </methodparam>
</xsl:template>

<xsl:template match="wsdl:part[starts-with(@type, 'xsd:')]" mode="methodparam">
    <methodparam>
        <type><xsl:value-of select="substring-after(@type, ':')"/></type>
        <parameter><xsl:value-of select="@name"/></parameter>
    </methodparam>
</xsl:template>

</xsl:stylesheet>
